#include <iostream>
#include <cstdlib>
#include <cstring>
#include <vector>

//////////////////////////////    Array    //////////////////////////////
int partition(std::vector<int>& A, int lo, int hi)
{
	int r = rand() % (hi - lo + 1) + lo;
	std::swap(A[r], A[hi]);
	int pivot = A[hi], i = lo;
	for (int j = lo; j < hi; ++j)
		if (A[j] <= pivot)
			std::swap(A[i++], A[j]);
	std::swap(A[i], A[hi]);
	return i;
}

int partition(std::vector<int>& A, int lo, int hi)
{
	int pivot = A[lo];
	while (lo < hi)
	{
		while (lo < hi && A[hi] >= pivot)
			--hi;
		A[lo] = A[hi];
		while (lo < hi && A[lo] <= pivot)
			++lo;
		A[hi] = A[lo];
	}
	A[lo] = pivot;
	return lo;
}

void quickSort(std::vector<int>& A, int lo, int hi)
{
	if (lo < hi)
	{
		int pivot = partition(A, lo, hi);
		quickSort(A, lo, pivot - 1);
		quickSort(A, pivot + 1, hi);
	}
}

/** multi-threading edition. */
void quickSort(std::vector<int>& A, int lo, int hi)
{
	if (lo < hi)
	{
		int pivot = partition(A, lo, hi);
		std::future<void> fut = std::async(std::launch::any, quickSort, std::ref(A), lo, pivot - 1);
		quickSort(A, pivot + 1, hi);
		fut.wait();
	}
}

void quickSorting(std::vector<int>& A)
{
	quickSort(A, 0, A.size()-1);
}

/** non-recursive edition. */
void quickSorting(std::vector<int>& A)
{
	if (A.size() < 2)
		return;
	int lo = 0, hi = static_cast<int>(A.size()) - 1;
	std::stack<int> S;
	S.push(lo); S.push(hi);
	while (!S.empty())
	{
		hi = S.top(); S.pop();
		lo = S.top(); S.pop();
		int pivot = partition(A, lo, hi);
		if (lo < pivot - 1)
		{
			S.push(lo);
			S.push(pivot - 1);
		}
		if (pivot + 1 < hi)
		{
			S.push(pivot + 1);
			S.push(hi);
		}
	}
}

//////////////////////////////    List    //////////////////////////////
ListNode* partition(ListNode *head, ListNode *tail)
{
	ListNode *p = head, *q = head;
	for (int pivot = head->val; q != tail; q = q->next)
	{
		if (q->val < pivot)
		{
			p = p->next;
			std::swap(p->val, q->val);
		}
	}
	std::swap(head->val, p->val);
	return p; // p is the pivot
}

void quickSort(ListNode *head, ListNode *tail)
{
	if (head != tail && head->next != tail)
	{
		ListNode *pivot = partition(head, tail);
		quickSort(head, pivot);
		quickSort(pivot->next, tail);
	}
}

ListNode* quickSortList(ListNode *head)
{
	quickSort(head, NULL);
	return head;
}

/** non-recursive edition. */
ListNode* quickSortList(ListNode *head)
{
	if (head == NULL || head->next == NULL)
		return head;
	ListNode *lo = head, *hi = NULL;
	std::stack<ListNode*> S;
	S.push(lo); S.push(hi);
	while (!S.empty())
	{
		hi = S.top(); S.pop();
		lo = S.top(); S.pop();
		ListNode *pivot = partition(lo, hi);
		if (lo != pivot && lo->next != pivot)
		{
			S.push(lo);
			S.push(pivot);
		}
		pivot = pivot->next;
		if (pivot != hi && pivot->next != hi)
		{
			S.push(pivot);
			S.push(hi);
		}
	}
	return head;
}
